##
# File:  ContentRequestReportDb.py
# Date:  15-Feb-2017  J. Westbrook
#
# Update:
#
##
"""
Fetch content and prepare report from PDBx content -

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import datetime
import logging
import sys
import time

#
#  -- supporting queries against only DA_INTERNAL in this service --
from wwpdb.utils.db.DaInternalSchemaDef import DaInternalSchemaDef
from wwpdb.utils.db.MyConnectionBase import MyConnectionBase
from wwpdb.utils.db.MyDbSqlGen import MyDbQuerySqlGen, MyDbConditionSqlGen
from wwpdb.utils.db.MyDbUtil import MyDbQuery
from wwpdb.utils.db.WorkflowSchemaDef import WorkflowSchemaDef

from wwpdb.apps.content_ws_server.content.ContentRequestReportIo import ContentRequestReportIo

#
logger = logging.getLogger()


class ContentRequestReportDb(MyConnectionBase):
    """
    Fetch content and prepare report from PDBx content -

    """

    def __init__(self, siteId, verbose=True, log=sys.stderr):
        super(ContentRequestReportDb, self).__init__(siteId=siteId, verbose=verbose, log=log)
        self.__verbose = verbose
        logger.info("Starting with siteId %r", siteId)
        self.__lfh = log
        #
        self.__crio = ContentRequestReportIo()

    def getContentTypeDef(self, contentType):
        return self.__crio.getContentDefinition(contentType)

    def getContentTypes(self):
        return self.__crio.getContentTypes()

    def extractContent(self, requestContentType):
        """Apply the input 'requestContentType' to the current database state -


        Example -
            {'req-emdb-summary-status-report': {'content': {'tablename': ['colname1','colname2','colname3'], },
                                               'type': 'rdbms',
                                               'resource': {'tablename': ('da_internal', 'da_internal_combine')},
                                               'conditions': {'tablename': {'colname': ('val', 'char', 'like')}
                                                          },
                                                }
             }
        """
        rD = {}
        try:
            cDef = self.getContentTypeDef(requestContentType)
            logger.debug("Content definition %r", cDef.items())
            logger.debug("Content keys definition %r", cDef["content"].keys())
            logger.debug("Content resource %r", cDef["resource"])
            #
            #  -- Note the str() filter here --
            myCategoryList = [str(c) for c in cDef["content"].keys()]
            myConditionList = [str(c) for c in cDef["conditions"].keys()]

            #
            if len(cDef) < 1:
                logger.error("Undefined/empty content definition")
                return rD

            #
            logger.debug("Category list in definition %r", myCategoryList)
            #
            for catName in myCategoryList:
                rD[catName] = []
                sList = cDef["content"][catName]
                myResource, myDatabase = cDef["resource"][catName]
                logger.debug("Resource %r database %r", myResource, myDatabase)
                if myResource.upper() in ["DA_INTERNAL", "STATUS"]:
                    if myDatabase in ["da_internal", "da_internal_prod", "da_internal_combine"]:
                        sDef = DaInternalSchemaDef(verbose=self.__verbose, log=sys.stderr, databaseName=myDatabase)
                    elif myDatabase in ["status"]:
                        sDef = WorkflowSchemaDef(verbose=self.__verbose, log=sys.stderr)
                    else:
                        sDef = {}
                else:
                    logger.error("Undefined resource %r database %r", myResource, myDatabase)
                    continue

                # tableIdList = sd.getTableIdList()
                # aIdList = sd.getAttributeIdList(tableId)
                sqlGen = MyDbQuerySqlGen(schemaDefObj=sDef, verbose=self.__verbose, log=sys.stderr)
                sTableIdList = []

                sTableIdList.append(catName.upper())
                for s in sList:
                    sqlGen.addSelectAttributeId(attributeTuple=(catName.upper(), s.upper()))

                if catName in myConditionList:
                    # {'entity_poly': {'entity_id': ('1', 'char', 'eq')},
                    sqlCondition = MyDbConditionSqlGen(schemaDefObj=sDef, verbose=self.__verbose, log=sys.stderr)
                    cndD = cDef["conditions"][catName]
                    cList = []
                    for k, v in cndD.items():
                        cList.append(((catName.upper(), k.upper()), v[2].upper(), (v[0].upper(), v[1].upper())))
                    for cTup in cList:
                        sqlCondition.addValueCondition(cTup[0], cTup[1], cTup[2])

                    sqlCondition.addTables(sTableIdList)
                    sqlGen.setCondition(sqlCondition)
                    #
                sqlS = sqlGen.getSql()
                logger.debug("SQL string\n %s\n\n", sqlS)
                sqlGen.clear()
                #
                rL = self.__processQuery(myResource, sList, sqlS)
                rD[catName] = rL
        except Exception as e:
            logger.exception("Database extraction failing for content type %r", requestContentType)
            logger.exception(e)
        #
        return rD

    def __processQuery(self, resourceName, sList, sqlS):
        """Process query  -"""
        startTime = time.time()
        #
        # config the db auth info
        self.setResource(resourceName=resourceName.upper())

        rL = []
        #
        try:
            self.openConnection()
            myQ = MyDbQuery(dbcon=self._dbCon, verbose=self.__verbose, log=self.__lfh)
            rowList = myQ.selectRows(queryString=sqlS)
            #
            if self.__verbose:
                logger.debug("Result length %d", len(rowList))
                # logger.debug("Row list %r" % rowList)
                #
            for row in rowList:
                d = {}
                for ii, s in enumerate(sList):
                    tt = row[ii]
                    try:
                        if isinstance(tt, datetime.datetime) or isinstance(tt, datetime.date):
                            tt = tt.isoformat()
                    except Exception as e:
                        logger.exception("Failing %r", tt)
                        logger.exception(e)
                    d[s] = tt
                rL.append(d)
            #
            self.closeConnection()
        except Exception as e:
            logger.exception("Failing for resource %r and query %r", resourceName, sqlS)
            logger.exception(e)

        endTime = time.time()
        logger.debug("Completed in (%.2f seconds)", endTime - startTime)
        return rL

    #
    def json_serial(self, obj):
        """JSON serializer for objects not serializable by default json code"""

        if isinstance(obj, datetime.datetime) or isinstance(obj, datetime.date):
            serial = obj.isoformat()
            return serial
        raise TypeError("Type not serializable")



##
# File:     doServiceRequest.wsgi
# Created:  10-Feb-2017
# Updates:
#     17-Feb-2017  jdw add auth filtering on content request type ---
##
"""
This top-level responder for web service requests ...

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import datetime
import logging
from webob import Request
from wwpdb.utils.config.ConfigInfo import getSiteId
from wwpdb.utils.ws_utils.ServiceResponse import ServiceResponse
from wwpdb.utils.ws_utils.TokenUtils import JwtTokenReader

from wwpdb.apps.content_ws_server.webapp.WebServiceApp import WebServiceApp

USEKEY = True
# Create logger
FORMAT = "[%(levelname)s]-%(module)s.%(funcName)s: %(message)s"
logging.basicConfig(format=FORMAT)
logger = logging.getLogger()
#
# Note:  Don't add another handler as this will cause apache to duplicate log records --
# ch = logging.StreamHandler()
# formatter = logging.Formatter('[%(levelname)s]-%(module)s.%(funcName)s: %(message)s')
# ch.setFormatter(formatter)
# logger.addHandler(ch)
#
logger.setLevel(logging.DEBUG)


class MyRequestApp(object):
    """Process request and response from WSGI server using WebOb Request/Response objects."""

    def __init__(self, serviceName="ContentServiceApp", authVerifyFlag=True):
        """ """
        self.__serviceName = serviceName
        self.__authVerifyFlag = authVerifyFlag
        #

    def __dumpRequest(self, request):
        outL = []
        outL.append("%s" % ("-" * 80))
        outL.append("Web server request data content:")
        try:
            outL.append("Host:            %s" % request.host)
            outL.append("Remote host:     %s" % request.remote_addr)
            outL.append("Path:            %s" % request.path)
            outL.append("Method:          %s" % request.method)
            outL.append("wwpdb-api-token: %s" % request.headers.get("wwpdb-api-token"))
            outL.append("Headers:         %r" % sorted(request.headers.items()))
            # outL.append("Environ:         %r" % sorted(request.environ.items()))
            outL.append("Query string:    %s" % request.query_string)
            outL.append("Parameter List:")
            for name, value in request.params.items():
                outL.append("  ++  Request parameter:    %s:  %r\n" % (name, value))
        except Exception as e:
            logger.exception("FAILING for service %s", self.__serviceName)
            logger.exception(e)
        return outL

    def __isHeaderApiToken(self, request):
        try:
            tok = request.headers.get("wwpdb-api-token")
            if tok and len(tok) > 20:
                return True
        except Exception as e:
            logger.exception(e)

        return False

    def __authVerify(self, authHeader, siteId, serviceUserId):
        """Check the validity of the input API access token - check for user id consistency .

        Return: True for success or False otherwise
        """
        tD = {"errorCode": 401, "errorMessage": "Token processing error", "token": None, "errorFlag": True}

        try:
            jtu = JwtTokenReader(siteId=siteId)
            tD = jtu.parseAuth(authHeader)
            if tD["errorFlag"]:
                return tD
            tD = jtu.parseToken(tD["token"])
            logger.debug("authVerify tD %r", tD)
            #
            if (len(serviceUserId) > 0) and (serviceUserId != str(tD["sub"])):
                tD["errorMessage"] = "Token user mismatch"
                tD["errorFlag"] = True
                tD["errorCode"] = 401
            #

            return tD
        except Exception as e:
            logger.exception("Failed site %r auth header %r", siteId, authHeader)
            logger.exception(e)
            return tD

    def __call__(self, environment, responseApplication):
        """Request callable entry point"""
        #
        myRequest = Request(environment)
        logger.debug("%s", "\n ++ ".join(self.__dumpRequest(request=myRequest)))
        #
        myParameterDict = {"request_host": [""], "wwpdb_site_id": [getSiteId()], "service_user_id": [""], "remote_addr": [""]}
        #
        try:
            #
            # Injected from the web server environment
            if "WWPDB_SITE_ID" in environment:
                myParameterDict["wwpdb_site_id"] = [environment["WWPDB_SITE_ID"]]

            if "HTTP_HOST" in environment:
                myParameterDict["request_host"] = [environment["HTTP_HOST"]]

            myParameterDict["remote_addr"] = [myRequest.remote_addr]
            #
            # Injected from the incoming request payload
            #
            for name, value in myRequest.params.items():
                if name not in myParameterDict:
                    myParameterDict[name] = []
                myParameterDict[name].append(value)
            myParameterDict["request_path"] = [myRequest.path.lower()]

        except Exception as e:
            logger.exception("Exception processing %s request parameters", self.__serviceName)
            logger.exception(e)

        ###
        # At this point we have everything needed from the request !
        ###
        # logger.debug("%s" % ("\n ++ ".join(self.__dumpRequest(request=myRequest))))
        # logger.debug("Parameter dict:\n%s" % '\n'.join(["  ++  %s: %r" % (k, v) for k, v in myParameterDict.items()]))
        try:
            ok = True
            apiTokFlag = self.__isHeaderApiToken(myRequest)
            logger.debug("Request API token flag: %s", apiTokFlag)
            if self.__authVerifyFlag and apiTokFlag:
                #  Verify API token -
                authD = self.__authVerify(myRequest.headers.get("wwpdb-api-token"), myParameterDict["wwpdb_site_id"][0], myParameterDict["service_user_id"][0])
                logger.debug("Authorization error flag is %r", authD["errorFlag"])
                if authD["errorFlag"]:
                    ok = False
                else:
                    myParameterDict["jwt-sub"] = [str(authD["sub"])]
                    myParameterDict["jwt-exp"] = [authD["exp"]]
                    myParameterDict["jwt-iat"] = [authD["iat"]]
                    myParameterDict["jwt-exp-ts"] = [datetime.datetime.utcfromtimestamp(authD["exp"]).strftime("%Y-%b-%d %H:%M:%S")]
                    myParameterDict["jwt-iat-ts"] = [datetime.datetime.utcfromtimestamp(authD["iat"]).strftime("%Y-%b-%d %H:%M:%S")]
                    myParameterDict["service_user_id"] = myParameterDict["jwt-sub"]
            else:
                myParameterDict["service_user_id"] = ["CONTENTWS_ANONYMOUS"]

            #  If this is a content request then -
            if ok:
                try:
                    if "request_content_type" in myParameterDict:
                        rOk = False
                        sId = str(myParameterDict["service_user_id"][0]).upper()
                        ct = str(myParameterDict["request_content_type"][0]).lower()
                        logger.info("Checking auth for %s and %s", sId, ct)
                        if sId.startswith("SASBDBWS_") and "sasbdb" in ct:
                            rOk = True
                        elif sId.startswith("EMDBWS_") and "emdb" in ct:
                            rOk = True
                        elif sId.startswith("CONTENTWS_") or sId.startswith("WWPDBWS_"):
                            rOk = True
                    else:
                        rOk = True
                except Exception as e:
                    logger.exception("Content request auth failure")
                    logger.exception(e)
                    rOk = False

                ok = rOk

            if ok:
                #        Check for sufficent data to continue
                if len(myParameterDict["service_user_id"][0]) > 0 and len(myParameterDict["wwpdb_site_id"][0]) > 0:
                    logger.debug("Invoking web service application")
                    myApp = WebServiceApp(parameterDict=myParameterDict)
                    sR = myApp.run()
                    myResponse = sR.getResponse()
                else:
                    logger.debug("Failed to invoke web service application")
                    sR = ServiceResponse(returnFormat="json")
                    sR.setError(statusCode=400, msg="Request missing input data or access key information")
                    myResponse = sR.getResponse()
            else:
                logger.debug("Authorization failure returning error status")
                sR = ServiceResponse(returnFormat="json")
                sR.setError(statusCode=authD["errorCode"], msg=authD["errorMessage"])
                myResponse = sR.getResponse()
        except Exception as e:
            logger.exception("Service request processing exception")
            logger.exception(e)
        #
        #
        #
        logger.debug("Request processing completed for service %s\n\n", self.__serviceName)
        ###
        return myResponse(environment, responseApplication)


##
##
application = MyRequestApp(serviceName="ContentServiceApp", authVerifyFlag=USEKEY)



from wwpdb.utils.config.ConfigInfo import getSiteId, ConfigInfo


def get_queue_name(site_id=None):
    if not site_id:
        site_id = getSiteId()
    cI = ConfigInfo(site_id)
    message_queue = cI.get("SITE_CONTENT_WS_MESSAGE_QUEUE")
    queue_name = message_queue if message_queue else "contentws_queue_{}".format(site_id)
    return queue_name


def get_routing_key():
    return "contentws_requests"


def get_exchange_name():
    return "biocurationws_exchange"


def get_exchange_topic():
    return "topic"



##
# File:     Register.py
# Created:  15-Feb-2017
# Updates:
##
"""
Off-line API registration -

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import argparse
import logging

from wwpdb.utils.ws_utils.TokenUtils import JwtTokenUtils

# Create logger
logger = logging.getLogger()
ch = logging.StreamHandler()
formatter = logging.Formatter("[%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
ch.setFormatter(formatter)
logger.addHandler(ch)
logger.setLevel(logging.INFO)


class Register(object):
    """Off-line API registration -"""

    def __init__(self):
        pass
        #

    def makeAccessToken(
        self,
        emailAddress,
        tokenPrefix="CONTENTWS",
        expireDays=30,
        tokenFileName="onedep_biocuration_apikey.jwt",
    ):
        """Test acquire new or existing token and write to disk"""
        try:
            tU = JwtTokenUtils(tokenPrefix=tokenPrefix)
            tokenId, jwtToken = tU.getToken(emailAddress, expireDays=expireDays)
            logging.info("For e-mail %r token ID %r is %r ", emailAddress, tokenId, jwtToken)
            tD = tU.parseToken(jwtToken)
            logging.info("token %r payload %r ", tokenId, tD)
            #
            with open(tokenFileName, "w") as outfile:
                outfile.write("%s" % jwtToken)
            logging.info("token written to %r ", tokenFileName)
            ok = True
        except Exception as e:
            logging.exception("Making token failed")
            logging.exception(e)
            ok = False
        return ok


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-d",
        "--debug",
        help="debugging",
        action="store_const",
        dest="loglevel",
        const=logging.DEBUG,
        default=logging.INFO,
    )
    parser.add_argument(
        "--emailAddress",
        help="email address",
        type=str,
        default="jdwestbrook@gmail.com",
    )
    parser.add_argument("--tokenPrefix", help="tokenPrefix", type=str, default="CONTENTWS")
    parser.add_argument("--expireDays", help="expireDays", type=int, default=30)

    args = parser.parse_args()
    logger.setLevel(args.loglevel)

    reg = Register()
    reg.makeAccessToken(
        emailAddress=args.emailAddress,
        tokenPrefix=args.tokenPrefix,
        expireDays=args.expireDays,
    )


if __name__ == "__main__":
    main()



##
# File:  ContentRequest.py
# Date:  14-Feb-2017  J. Westbrook
#
# Update:
#   16-Feb-2017 jdw add summary content request support --
#   14-Mar-2017 jdw remove some unused code and uncecessary file checks
##
"""
Manage invoking content request for web service -

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import copy
import os.path

import json
import logging
import os
import time

#
from wwpdb.utils.config.ConfigInfo import ConfigInfo, getSiteId
from wwpdb.utils.ws_utils.ServiceDataStore import ServiceDataStore
from wwpdb.utils.ws_utils.ServiceHistory import ServiceHistory

from wwpdb.apps.content_ws_server.content.ContentRequestPolicyFilter import ContentRequestPolicyFilter
from wwpdb.apps.content_ws_server.content.ContentRequestProxyReportPdbx import ContentRequestProxyReportPdbx
from wwpdb.apps.content_ws_server.content.ContentRequestReportDb import ContentRequestReportDb

#
from wwpdb.apps.content_ws_server.content.ContentRequestReportPdbx import ContentRequestReportPdbx

#
logger = logging.getLogger()


class ContentRequest(object):
    """
    Manage various request for entry and summary level content.

    """

    def __init__(self):
        self.__siteId = getSiteId(defaultSiteId=None)
        self.__cI = ConfigInfo(self.__siteId)
        logger.info("Starting with siteId %r", self.__siteId)
        self.__topSessionPath = self.__cI.get("SITE_WEB_APPS_TOP_SESSIONS_PATH")
        logger.info("Starting with session path %r", self.__topSessionPath)
        self.__sessionPath = None
        self.__sdsPrefix = None
        self.__sds = None
        self.__debugPayload = False
        #
        self.__pD = {}

    def setup(self, pD):
        logger.debug("Content request input payload %r", pD)
        try:
            self.__sessionPath = pD["session_path"]
            self.__sdsPrefix = pD["session_store_prefix"]
            self.__sds = ServiceDataStore(sessionPath=self.__sessionPath, prefix=self.__sdsPrefix)
            self.__sds.set("status", "running")
            #
            #  JDW not used
            # self.__sD = self.__sds.getDictionary()
            # logger.info("Session dictionary %r", self.__sD)
            #
            # logger.info("Preparing copy of dictionary")
            self.__pD = copy.deepcopy(pD)
            #
            if self.__debugPayload:
                logger.debug("Parameter dictionary copy %r", self.__pD)
            return True
        except Exception as e:
            logger.exception("Failing %s", str(e))

        return False

    def run(self):
        """Service execution method"""
        iD = {}
        #
        testMode = self.__pD.get("worker_test_mode", False)
        successFlag = False
        #
        if testMode:
            try:
                wSecs = int(self.__pD.get("worker_test_duration", 10))
                logger.info("Running in test mode with duration %d seconds", wSecs)
                time.sleep(wSecs)
            except Exception as e:
                logger.exception(e)

            try:
                ct = self.__pD["request_content_type"]
                fp = self.__pD["report_path"]
                fn = self.__pD["report_file"]
                with open(fp, "w") as ofh:
                    ofh.write("DUMMY")
                iD[ct] = (fn, "data")
                successFlag = True
            except Exception as e:
                logger.exception("Mock execution file update failing %s", str(e))
            #
            logger.info("Test mode service completed")
        else:
            #
            successFlag = self.__run(self.__pD)
            # Add the output files to the session store -
            ct = self.__pD["request_content_type"]
            fp = self.__pD["report_path"]
            fn = self.__pD["report_file"]
            #
            # JDW  - Move down
            # if os.access(fp, os.R_OK):
            #    iD[ct] = (fn, 'data')
            #
            # Add an additional existence tests for key report files -
            #
            if successFlag:
                fp = self.__pD["report_path"]
                if not os.access(fp, os.R_OK):
                    successFlag = False
                else:
                    successFlag = True
                    iD[ct] = (fn, "data")
        #
        if successFlag:
            iD["status"] = "completed"
        else:
            iD["status"] = "failed"
        #
        try:
            #
            # Update service activity tracking
            sH = ServiceHistory(historyPath=self.__pD["session_history_path"])
            sH.add(sessionId=self.__pD["session_id"], statusOp=iD["status"])
            logger.info("Updated service history store with %r", iD["status"])
            #
        except Exception as e:
            logger.exception("Failed to update session tracking history status %r", iD)
            logger.exception(e)

        try:
            #  Update session store -
            logger.info("Updated session store with %r", iD)
            self.__sds.updateAll(iD)
            #
            tStatus = self.__sds.get("status")
            logger.info("Read session store status as  %r", tStatus)
        except Exception as e:
            logger.exception("Failed to update session status %r", iD)
            logger.exception(e)
        #
        logger.info("Updated session store with: %r", iD)
        return True

    def __run(self, pD):
        """Create entry level and summary content reports  --"""
        ok = False
        try:
            #
            sessionPath = pD["session_path"]
            cI = ConfigInfo()
            siteId = cI.get("SITE_PREFIX")
            #
            dataSetId = pD["request_dataset_id"]
            contentType = pD["request_content_type"]
            reportFile = pD["report_file"]
            reportPath = pD["report_path"]
            proxyReportUrl = pD.get("session_proxy_url")

            if proxyReportUrl and contentType.startswith("report-entry-"):
                logger.debug("Forwarding request to another server for an entry")
                logger.debug("pD is %r", pD)
                formatType = pD.get("request_format_type")

                cr = ContentRequestProxyReportPdbx()
                # Need to test for rejection - id not found and forward back errors
                status = cr.retrieveProxyReport(dataSetId, proxyReportUrl, contentType, formatType, reportPath)

                ok = status

            elif contentType.startswith("report-entry-"):
                cr = ContentRequestReportPdbx()
                ctypeL = cr.getContentTypes()
                if contentType in ctypeL:
                    logger.debug("Processing content definition %r", contentType)
                    logFilePath = os.path.join(self.__sessionPath, dataSetId + " -parser.log")
                    pdbxFilePath = pD["session_pdbx_file_path"]
                    #
                    rD = cr.extractContent(pdbxFilePath, logFilePath, contentType)
                    cF = ContentRequestPolicyFilter()
                    # Filter content based on contentType policies
                    rD = cF.filterContent(contentType, rD)
                    if self.__debugPayload:
                        logger.debug("File content %r", rD)
                    ss = json.dumps(rD)
                    if self.__debugPayload:
                        logger.info("JSON serialized result %r", ss)
                    with open(reportPath, "w") as ofh:
                        ofh.write(ss)
                    ok = True
            elif contentType.startswith("report-summary-"):
                site = self.__siteId
                qs = pD["query_site"]
                if qs is not None:
                    site = qs
                cr = ContentRequestReportDb(siteId=site)
                ctL = cr.getContentTypes()
                if contentType in ctL:
                    rD = cr.extractContent(contentType)
                    logger.info("Database content length %r", len(rD))
                    ss = json.dumps(rD)
                    logger.info("JSON serialized result length %r", len(ss))
                    with open(reportPath, "w") as ofh:
                        ofh.write(ss)
                    ok = True
            else:
                ok = False
            #
            logger.info(" - Site Id: %r", siteId)
            logger.info(" - Session path: %r", sessionPath)
            logger.info(" - Dataset Id:   %r", dataSetId)
            logger.info(" - Content Type: %r", contentType)
            logger.info(" - Report file: %r", reportFile)
            logger.info(" - Report path: %r", reportPath)
            logger.info(" - Return status: %r", ok)

            return ok
        except Exception as e:
            logger.exception("Failing content request runner method %s", str(e))

        return False
        #



import os
import pathlib


def get_content_definition_file_path():
    my_folder = pathlib.Path(__file__).parent.absolute()
    return os.path.join(my_folder, get_content_definition_file())


def get_content_definition_file():
    return "ws_content_type_definitions.json"



#
# File:  ContentRequestServiceHandler (adapted from DetachedMessageConsumerExample.py)
# Date:  13-Feb-2017
#
#  Controlling wrapper for content request service consumer client -
#
#  Updates:
#  18-Feb-2017  jdw switch to using default ampq connection url
#
##

import platform

import json
import logging
import os
import sys
import time
from argparse import ArgumentParser as ArgParser
from wwpdb.utils.config.ConfigInfo import ConfigInfo, getSiteId
from wwpdb.utils.detach.DetachedProcessBase import DetachedProcessBase
from wwpdb.utils.message_queue.MessageConsumerBase import MessageConsumerBase
from wwpdb.utils.message_queue.MessageQueueConnection import MessageQueueConnection

from wwpdb.apps.content_ws_server.content.ContentRequest import ContentRequest
from wwpdb.apps.content_ws_server.message_queue.MessageQueue import get_queue_name, get_exchange_name, get_exchange_topic, get_routing_key

logger = logging.getLogger()
logging.basicConfig(
    level=logging.DEBUG,
    format="\n%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s",
)


class MessageConsumer(MessageConsumerBase):
    # def __init__(self, amqpUrl):
    #     super(MessageConsumer, self).__init__(amqpUrl)

    def workerMethod(self, msgBody, deliveryTag=None):
        try:
            # logger.debug("Message body %r" % msgBody)
            pD = json.loads(msgBody)
        except Exception as e:
            logger.error("Message format error - discarding")
            logger.exception(e)
            return False
        #
        successFlag = True
        try:
            # logger.info("Message body %r", pD)
            v = ContentRequest()
            v.setup(pD)
            v.run()
        except Exception as e:
            logger.exception("Failed service execution with message %r", pD)
            logger.exception(e)

        return successFlag


class MessageConsumerWorker(object):
    def __init__(self):
        self.__setup()

    def __setup(self):
        mqc = MessageQueueConnection()
        url = mqc._getDefaultConnectionUrl()  # pylint: disable=protected-access
        self.__mc = MessageConsumer(amqpUrl=url)
        self.__mc.setQueue(queueName=get_queue_name(), routingKey=get_routing_key())
        self.__mc.setExchange(exchange=get_exchange_name(), exchangeType=get_exchange_topic())
        #

    def run(self):
        """Run async consumer"""
        startTime = time.time()
        logger.info("Starting ")
        try:
            try:
                logger.info("Run consumer worker starts")
                self.__mc.run()
            except KeyboardInterrupt:
                self.__mc.stop()
        except Exception as e:
            logger.exception("MessageConsumer failing")
            logger.exception(e)

        endTime = time.time()
        logger.info("Completed (%.3f seconds)", endTime - startTime)

    def suspend(self):
        logger.info("Suspending consumer worker... ")
        self.__mc.stop()


class MyDetachedProcess(DetachedProcessBase):
    """This class implements the run() method of the DetachedProcessBase() utility class.

    Illustrates the use of python logging and various I/O channels in detached process.
    """

    def __init__(
        self,
        pidFile="/tmp/DetachedProcessBase.pid",
        stdin=os.devnull,
        stdout=os.devnull,
        stderr=os.devnull,
        wrkDir="/",
        gid=None,
        uid=None,
    ):
        super(MyDetachedProcess, self).__init__(
            pidFile=pidFile,
            stdin=stdin,
            stdout=stdout,
            stderr=stderr,
            wrkDir=wrkDir,
            gid=gid,
            uid=uid,
        )
        self.__mcw = MessageConsumerWorker()

    def run(self):
        logger.info("STARTING detached run method")
        self.__mcw.run()

    def suspend(self):
        logger.info("SUSPENDING detached process")
        try:
            self.__mcw.suspend()
        except Exception as e:
            logger.exception(e)


def main():
    # adding a conservative permission mask for this
    # os.umask(0o022)
    #
    siteId = getSiteId(defaultSiteId=None)
    cI = ConfigInfo(siteId)

    #    topPath = cI.get('SITE_WEB_APPS_TOP_PATH')
    topSessionPath = cI.get("SITE_WEB_APPS_TOP_SESSIONS_PATH")

    #
    myFullHostName = platform.uname()[1]
    myHostName = str(myFullHostName.split(".")[0]).lower()
    #
    wsLogDirPath = os.path.join(topSessionPath, "ws-logs")
    if not os.path.exists(wsLogDirPath):
        os.makedirs(wsLogDirPath)

    #  Setup logging  --
    now = time.strftime("%Y-%m-%d", time.localtime())

    description = "Content request service handler"
    parser = ArgParser(description=description)

    parser.add_argument(
        "--start",
        default=False,
        action="store_true",
        dest="startOp",
        help="Start consumer client process",
    )
    parser.add_argument(
        "--stop",
        default=False,
        action="store_true",
        dest="stopOp",
        help="Stop consumer client process",
    )
    parser.add_argument(
        "--restart",
        default=False,
        action="store_true",
        dest="restartOp",
        help="Restart consumer client process",
    )
    parser.add_argument(
        "--status",
        default=False,
        action="store_true",
        dest="statusOp",
        help="Report consumer client process status",
    )

    parser.add_argument(
        "--debug",
        default=1,
        type=int,
        dest="debugLevel",
        help="Debug level (default: 1 [0-3]",
    )
    parser.add_argument(
        "--instance",
        default=1,
        type=int,
        dest="instanceNo",
        help="Instance number [1-n]",
    )
    #
    # (options, args) = parser.parse_args()

    options = parser.parse_args()
    if isinstance(options, tuple):
        args = options[0]
    else:
        args = options
    del options

    #
    pidFilePath = os.path.join(wsLogDirPath, myHostName + "_" + str(args.instanceNo) + ".pid")
    stdoutFilePath = os.path.join(wsLogDirPath, myHostName + "_" + str(args.instanceNo) + "_stdout.log")
    stderrFilePath = os.path.join(wsLogDirPath, myHostName + "_" + str(args.instanceNo) + "_stderr.log")
    wfLogFilePath = os.path.join(wsLogDirPath, myHostName + "_" + str(args.instanceNo) + "_" + now + ".log")
    #
    logger = logging.getLogger(name="root")  # pylint: disable=redefined-outer-name
    logging.captureWarnings(True)
    formatter = logging.Formatter("%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
    handler = logging.FileHandler(wfLogFilePath)
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    #
    lt = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
    #
    if args.debugLevel > 2:
        logger.setLevel(logging.DEBUG)
        logger.debug("Setting DEBUG logging")
    elif args.debugLevel > 0:
        logger.setLevel(logging.INFO)
        logger.debug("Setting INFO logging")
    else:
        logger.setLevel(logging.ERROR)
    #
    #
    myDP = MyDetachedProcess(
        pidFile=pidFilePath,
        stdout=stdoutFilePath,
        stderr=stderrFilePath,
        wrkDir=wsLogDirPath,
    )

    if args.startOp:
        sys.stdout.write("+DetachedMessageConsumer() starting consumer service at %s\n" % lt)
        logger.info("DetachedMessageConsumer() starting consumer service at %s", lt)
        myDP.start()
    elif args.stopOp:
        sys.stdout.write("+DetachedMessageConsumer() stopping consumer service at %s\n" % lt)
        logger.info("DetachedMessageConsumer() stopping consumer service at %s", lt)
        myDP.stop()
    elif args.restartOp:
        sys.stdout.write("+DetachedMessageConsumer() restarting consumer service at %s\n" % lt)
        logger.info("DetachedMessageConsumer() restarting consumer service at %s", lt)
        myDP.restart()
    elif args.statusOp:
        sys.stdout.write("+DetachedMessageConsumer() reporting status for consumer service at %s\n" % lt)
        sys.stdout.write(myDP.status())
    else:
        pass


if __name__ == "__main__":
    main()



##
# File:  ContentRequestReportPdbx.py
# Date:  14-Feb-2017  J. Westbrook
#
# Update:
#     16-Feb-2017  jdw add limited condition filters -
##
"""
Fetch content and prepare report from PDBx content -

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import logging
import sys
import time
from mmcif.io.IoAdapterCore import IoAdapterCore

from wwpdb.apps.content_ws_server.content.ContentRequestReportIo import ContentRequestReportIo

#
logger = logging.getLogger()


class ContentRequestReportPdbx(object):
    """
    Fetch content and prepare report from PDBx content -

    """

    def __init__(self, verbose=True):
        self.__verbose = verbose
        #
        self.__crio = ContentRequestReportIo()
        logger.info("Starting ContentRequestReportPdbx")

    def getContentTypeDef(self, contentType):
        return self.__crio.getContentDefinition(contentType)

    def getContentTypes(self):
        return self.__crio.getContentTypes()

    def readFilePdbx(self, filePath, logFilePath, catNameList=None):
        """Read selected categories from PDBx file"""
        startTime = time.time()
        containerList = []
        try:
            io = IoAdapterCore(verbose=self.__verbose, log=sys.stderr)
            if catNameList and len(catNameList) > 0:
                containerList = io.readFile(str(filePath), selectList=catNameList, logFilePath=str(logFilePath))
            else:
                containerList = io.readFile(str(filePath), logFilePath=str(logFilePath))
            #
            logger.info("Read %d data blocks from %r", len(containerList), filePath)

        except Exception as e:
            logger.exception("Read failing for %r", filePath)
            logger.exception(e)

        endTime = time.time()
        logger.info("Completed in (%.2f seconds)", endTime - startTime)
        return containerList

    def __cmpfunc(self, v, target, myType, myOp):
        """Internal function to compare simple types -"""
        if myType in ["string", "char"]:
            vv = str(v)
        elif myType == "int":
            vv = int(str(v))
        elif myType in ["float", "double"]:
            vv = float(str(v))
        else:
            vv = str(v)
        #
        if type in ["string", "char", "float", "double", "int"]:
            if myOp in ["eq"]:
                return vv == target
            elif myOp in ["gt"]:
                return vv > target
            elif myOp in ["ge"]:
                return vv >= target
            elif myOp in ["lt"]:
                return vv < target
            elif myOp in ["le"]:
                return vv <= target
            else:
                return False
        #
        return False

    def extractContent(self, pdbxFilePath, logFilePath, requestContentType):
        """Apply the input 'requestContentType' to the content of the input PDBx data file -"""
        rD = {}
        try:
            cDef = self.getContentTypeDef(requestContentType)
            logger.info("Content definition %r", cDef.items())
            logger.info("Content keys definition %r", cDef["content"].keys())
            # Note the str() filter here -
            myCategoryList = [str(c) for c in cDef["content"].keys()]
            myConditionList = [str(c) for c in cDef["conditions"].keys()]
            #
            if len(cDef) < 1:
                return rD
            logger.info("Category list in definition %r", myCategoryList)
            myContainerList = self.readFilePdbx(pdbxFilePath, logFilePath, myCategoryList)
            for container in myContainerList:
                catNameList = container.getObjNameList()
                for catName in myCategoryList:
                    if catName in catNameList:
                        rD[catName] = []
                        catSel = cDef["content"][catName]
                        if catName in myConditionList:
                            cndD = cDef["conditions"][catName]
                        else:
                            cndD = {}
                        cObj = container.getObj(catName)
                        for ii in range(0, cObj.getRowCount()):
                            od = {}
                            dd = cObj.getRowAttributeDict(ii)
                            for k, v in dd.items():
                                # Check for a condition -
                                if k in cndD:
                                    fD = cndD[k]
                                    ok = self.__cmpfunc(v, fD[0], fD[1], fD[2])
                                    if not ok:
                                        continue
                                #
                                if k in catSel:
                                    od[k] = v
                            rD[catName].append(od)
        except Exception as e:
            logger.exception("Extraction processing failing for %r content type %r", pdbxFilePath, requestContentType)
            logger.exception(e)
        #
        return rD



##
# File:  ContentRequestReportIo.py
# Date:  14-Feb-2017  J. Westbrook
#
# Update:
#
##
"""
     Manage fetching and storing  content type definitions.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import datetime
import logging

from wwpdb.apps.content_ws_server.content_definitions.ContentDefintions import get_content_definition_file_path

try:
    import json
except ImportError:
    import simplejson as json
#
from oslo_concurrency import lockutils
from wwpdb.utils.config.ConfigInfo import ConfigInfo, getSiteId

#
logger = logging.getLogger()


class ContentRequestReportIo(object):
    """
    Manage fetching and storing content type definitions.

    """

    def __init__(self):
        self.__siteId = getSiteId(defaultSiteId=None)
        self.__cI = ConfigInfo(self.__siteId)
        logger.info("Starting with siteId %r", self.__siteId)
        self.__D = None
        #
        self.__lockDirPath = self.__cI.get("SITE_SERVICE_REGISTRATION_LOCKDIR_PATH", ".")
        lockutils.set_defaults(self.__lockDirPath)

    def __setup(self):
        if self.__D is None:
            self.__D = self.__readContentDefinitionDictionary()

    def getContentDefinition(self, contentType):
        self.__setup()
        if contentType and contentType in self.__D:
            return self.__D[contentType]
        else:
            return {}

    def getContentTypes(self):
        """Return a list of defined content types -"""
        self.__setup()
        if self.__D is not None:
            return self.__D.keys()
        else:
            return []

    def __get_content_definition_file(self):
        """
        helper method for getting path to content definition file
        :return:
        """
        # return self.__cI.get('SITE_WS_CONTENT_TYPE_DEFINITION_FILE_PATH')
        return get_content_definition_file_path()

    def __readContentDefinitionDictionary(self):
        """Read the dictionary containing web service content type definitions.

        Returns: d[<content_type>] = {categoryName: [at1,at2,...], ...}  or a empty dictionary.
        """
        fp = self.__get_content_definition_file()
        try:
            with open(fp, "r") as infile:
                return json.load(infile)
        except Exception as e:
            logger.info("Failed reading json resource file %s", fp)
            logger.exception(e)

        return {}

    @lockutils.synchronized("wscontenttypedef.exceptionfile-lock", external=True)
    def writeContentDefinitionDictionary(self, contentDefD, backup=True):
        """Write the dictionary containing web service content type definitions.

        Returns: True for success or False otherwise
        """
        fp = self.__get_content_definition_file()

        try:
            if backup:
                bp = fp + datetime.datetime.now().strftime("-%Y-%m-%d-%H-%M-%S")
                d = self.__readContentDefinitionDictionary()
                with open(bp, "w") as outfile:
                    json.dump(d, outfile, indent=4)
            #
            with open(fp, "w") as outfile:
                json.dump(contentDefD, outfile, indent=4)
            return True
        except Exception as e:
            logger.exception("Failed writing json resource file %s -- %s", fp, str(e))

        return False



##
# File:  ContentServiceAppWorker.py
# Date:  8-Feb-2017  J.Westbrook
#
# Updates:
#   18-Feb-2017 jdw Change search approach for model file -
#   15-Mar-2017 ep  For model file, invoke DepUI to produce model
#   16-Mar-2017 jdw Change status tracking to avoid collisions
##
"""
Manage web request and response processing for miscellaneous annotation tasks.

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import glob

import datetime
import json
import logging
import os
import signal
import time
from subprocess import Popen, PIPE
from wwpdb.io.file.DataExchange import DataExchange
from wwpdb.io.locator.PathInfo import PathInfo
from wwpdb.utils.config.ConfigInfo import getSiteId
from wwpdb.utils.config.ConfigInfoDataSet import ConfigInfoDataSet
from wwpdb.utils.message_queue.MessagePublisher import MessagePublisher
from wwpdb.utils.ws_utils.ServiceSessionState import ServiceSessionState
from wwpdb.utils.ws_utils.ServiceUploadUtils import ServiceUploadUtils
from wwpdb.utils.ws_utils.ServiceUtilsMisc import getMD5
from wwpdb.utils.ws_utils.ServiceWorkerBase import ServiceWorkerBase

from wwpdb.apps.content_ws_server.message_queue.MessageQueue import get_queue_name, get_routing_key, get_exchange_name

logger = logging.getLogger(__name__)


class ContentServiceAppWorker(ServiceWorkerBase):
    def __init__(self, reqObj=None, sessionDataPrefix=None):
        """
        Worker methods for annotation tasks.

        Performs URL - application mapping and application launching
        for annotation tasks module.

        """
        super(ContentServiceAppWorker, self).__init__(reqObj=reqObj, sessionDataPrefix=sessionDataPrefix)
        #
        #  URL to method mapping -----  the service names are case insensitive --
        #
        self.__appPathD = {
            "/contentws/dump": "_dumpOp",
            "/contentws/session": "_sessionOp",
            "/contentws/upload": "_uploadFileOp",
            "/contentws/input_file": "_uploadFileOp",
            "/contentws/sessioninfo": "_sessionInfoOp",
            "/contentws/session_status": "_sessionInfoOp",
            "/contentws/download": "_downloadOp",
            "/contentws/output_file": "_uploadFileOp",
            "/contentws/submit": "_submitReportOp",
            "/contentws/entry_content": "_submitContentRequestOp",
            "/contentws/summary_content": "_submitContentRequestOp",
            "/contentws/index": "_indexOp",
            "/contentws/session_index": "_indexOp",
            "/contentws/session_activity": "_activityOp",
            "/contentws/activity": "_activityOp",
        }
        self.addServices(self.__appPathD)
        #

    def run(self, reqPath=None):
        #
        inpRequestPath = reqPath if reqPath else self._reqObj.getRequestPath()
        #
        # First pull off the REST style URLS --
        #
        #  /contentws/report/X_XXXXXX
        #
        logger.debug("Request prefix %s", self._reqObj.getRequestPathPrefix())
        logger.debug("Request path   %s", self._reqObj.getRequestPath())
        logger.debug("Request inp path %s", inpRequestPath)
        #
        if inpRequestPath.startswith("/contentws/report/d_"):
            rFields = inpRequestPath.split("/")
            self._reqObj.setValue("idcode", rFields[4].upper())
            requestPath = "/review/report"
        else:
            requestPath = inpRequestPath
        #
        return self._run(requestPath)

    def _dumpOp(self):
        """Dump the request object dictionary ----"""
        logger.debug("Starting now")
        sst = ServiceSessionState()
        rD = self._reqObj.getDictionary()
        sst.setAppDataDict(rD, format="json")
        logger.debug("Completed")
        return sst

    def _sessionInfoOp(self):
        logger.debug("Starting now")
        sst = ServiceSessionState()
        #            join an existing session -
        ok = self._getSession(new=False, useContext=True, contextOverWrite=True)
        if not ok:
            sst.setServiceCompletionFlag(ok)
            sst.setServiceError(msg="Session acquire failed")
        else:
            sst.setServiceCompletionFlag(ok)
            sD = self._getSessionStoreDict()
            if "session_history" in sD:
                shL = sD["session_history"]
            else:
                shL = []
            if "status" in sD:
                status = sD["status"]
            else:
                status = "unknown"
            sst.setAppDataDict(
                {
                    "session_id": self._sessionId,
                    "status": status,
                    "session_history": shL,
                },
                format="json",
            )
        #
        logger.debug("Completed")
        return sst

    def _sessionOp(self):
        """Create a new session -

        :Returns: sst service session state object

        """
        logger.debug("Starting now")
        sst = ServiceSessionState()
        #            Create a new session -
        ok = self._getSession(new=True, useContext=True, contextOverWrite=True)
        if not ok:
            sst.setServiceCompletionFlag(ok)
            sst.setServiceError(msg="Session creation failed")
        else:
            self._trackServiceStatus("created")
            self._setSessionStoreValue("status", "newsession")
            sst.setServiceCompletionFlag(ok)
            sst.setAppDataDict({"session_id": self._sessionId}, format="json")
        #
        logger.debug("Completed")
        return sst

    def _uploadFileOp(self):
        """Upload callback method -- for model and experimental files -

        Copy input model and experimental data files to the current session directory and
        return file name and entry id details to the caller.

        """

        try:
            logger.debug("Starting now")
            sst = ServiceSessionState()
            #            join an existing session -
            ok = self._getSession(new=False, useContext=True, contextOverWrite=True)
            if not ok:
                sst.setServiceError(msg="Session acquire failed")
            else:
                suu = ServiceUploadUtils(reqObj=self._reqObj)
                if not suu.isFileUpload():
                    sst.setServiceError(msg="No input file in request ")
                else:
                    fileName = suu.copyToSession(fileTag="file")
                    if fileName:
                        sst.setServiceCompletionFlag(True)
                        sst.setServiceStatusText("Upload successful")
                        #
                        contentType = self._reqObj.getValue("content_type")
                        fileFormat = self._reqObj.getValue("file_format")
                        self._setSessionStoreValue(contentType, (fileName, fileFormat))
                        #
                        sD = self._getSessionStoreDict()
                        if "session_history" in sD:
                            shL = sD["session_history"]
                        else:
                            shL = []
                        sst.setAppDataDict(
                            {"session_id": self._sessionId, "session_history": shL},
                            format="json",
                        )
                        self._setSessionStoreValue("status", "uploading")
                        # self._trackServiceStatus('uploading', {'file': fileName})
                        self._trackServiceStatus("uploading", file=fileName)
                    else:
                        sst.setServiceError(msg="File upload failed")
        except Exception as e:
            logger.exception("FAILING")
            logger.exception(e)
            sst = ServiceSessionState()
            sst.setServiceError(msg="File upload failed")

        return sst

    def __fetchModelFile(self, siteId, entryId):
        """Fetch model file first from archive then from the deposit file source ..."""
        generateModel = True

        pdbxFilePath = None
        # Sanity check
        if len(entryId) > 30:
            return None
        try:
            logger.debug("siteId %r ", siteId)
            # note explict use of siteId here --
            dx = DataExchange(
                reqObj=self._reqObj,
                depDataSetId=entryId,
                fileSource="archive",
                siteId=siteId,
                verbose=True,
            )
            fl = dx.getContentTypeFileList(fileSource="archive", contentTypeList=["model"])
            if len(fl) > 0:
                pdbxFilePath = dx.fetch(contentType="model", formatType="pdbx", version="latest")
            elif generateModel:
                pdbxFilePath = self.__depuiGenerateModelFile(siteId, entryId)

            # Fallback on deposit directory
            if not pdbxFilePath:
                dx = DataExchange(
                    reqObj=self._reqObj,
                    depDataSetId=entryId,
                    fileSource="deposit",
                    siteId=siteId,
                    verbose=True,
                )
                pdbxFilePath = dx.fetch(contentType="model", formatType="pdbx", version="latest")

        except Exception as e:
            logger.exception("Fetch model failing %r", entryId)
            logger.exception(e)
        return pdbxFilePath

    def __depuiGenerateModelFile(self, siteId, entryId):
        """Uses code in the deposition system to generate a model file in the session directory.
        returns path to filename or None if it fails
        """
        logger.debug("generate %r from depui", entryId)
        sessDir = self._reqObj.getSessionObj().getPath()

        # Create script to run. Django will need WWPDB_SITE_ID set before invoking
        cmdPy = os.path.join(sessDir, "generate.py")
        fOut = open(cmdPy, "w")
        fOut.write("#!/usr/bin/env python\n")
        fOut.write("import os, sys\n")
        # Setup the Django environment
        fOut.write('os.environ.setdefault("DJANGO_SETTINGS_MODULE", "wwpdb.apps.deposit.settings")\n')
        fOut.write("import django\n")
        fOut.write("django.setup()\n")
        fOut.write("from wwpdb.apps.deposit.depui.generate_model import generate_model\n")
        fOut.write('generate_model(depID="%s", sessionDir = "%s").write_out_cif_from_depui()' % (entryId, sessDir))
        fOut.close()
        cmdfile = os.path.join(sessDir, "generate.sh")
        fOut = open(cmdfile, "w")
        fOut.write("#!/bin/sh\n")
        fOut.write("export WWPDB_SITE_ID=%s\n" % siteId)
        fOut.write("export PYTHONPATH=$TOP_WWPDB_PYTHON_DIR/wwpdb/apps/deposit:$PYTHONPATH\n")
        fOut.write("python %s\n" % cmdPy)
        fOut.close()
        os.chmod(cmdfile, 0o777)

        logfile = os.path.join(sessDir, "generate.log")

        retCode = self.__runTimeout(cmdFile=cmdfile, logFile=logfile, timeout=30)

        logger.debug("response from depui %r", retCode)
        if retCode != 0:
            return None
        pI = PathInfo(siteId=self._siteId, sessionPath=sessDir, verbose=True)
        pdbxFilePath = pI.getModelPdbxFilePath(dataSetId=entryId, fileSource="session")
        # DepUI can generate file even if deposition non-existant!! Make sure minimal size....
        if os.access(pdbxFilePath, os.R_OK):
            statinfo = os.stat(pdbxFilePath)
            if statinfo.st_size > 100:
                return pdbxFilePath

        return None

    def _submitContentRequestOp(self):
        """Submit entry content service request  -"""
        logger.debug("Content request method starting now")
        sst = ServiceSessionState()
        #            Join an existing session -
        ok = self._getSession(new=False, useContext=True, contextOverWrite=True)
        if not ok:
            sst.setServiceCompletionFlag(ok)
            sst.setServiceError(msg="Session acquire failed")
        else:
            sD = self._getSessionStoreDict()
            ok = False
            pD = {}
            #
            #   Check for a test request
            #
            pD["worker_test_mode"] = self._reqObj.getValueOrDefault("worker_test_mode", default=False)
            pD["worker_test_duration"] = self._reqObj.getValueOrDefault("worker_test_duration", default=1)
            pD["exp_method"] = self._reqObj.getValueOrDefault("exp_method", default="unassigned")
            #
            pD["request_dataset_id"] = self._reqObj.getValueOrDefault("request_dataset_id", default="unassigned")
            pD["request_content_type"] = self._reqObj.getValueOrDefault("request_content_type", default="unassigned")
            pD["request_format_type"] = self._reqObj.getValueOrDefault("request_format_type", default="unassigned")
            #
            entryId = pD["request_dataset_id"]
            contentType = pD["request_content_type"]
            formatType = pD["request_format_type"]
            #
            # First check if we are handling this entry... If
            # config variable not set - will return ['None']
            siteCoverage = [x.strip() for x in str(self._cI.get("SITE_WS_CONTENT_SITE_COVERAGE")).split(",")]

            # Content type
            if entryId != "unassigned" and contentType.startswith("report-entry-"):
                ok = True
                fName = entryId + "_" + contentType + "." + formatType
                logger.debug("Entry content type %r format type %r and dataset id %r status %r", contentType, formatType, entryId, ok)

                cIDS = ConfigInfoDataSet()
                siteId = cIDS.getSiteId(entryId)

                # On development server
                if siteId == "UNASSIGNED":
                    siteId = self._siteId

                logger.debug("Entry siteid is %r", siteId)

                logger.debug("Entry %r my site %r cover %r", siteId, self._siteId, siteCoverage)
                if siteId != self._siteId and siteId not in siteCoverage:
                    logger.debug("Entry %r resides on another site %r", entryId, siteId)

                    # Simple lookup if we can redirect
                    siteMap = self._cI.get("PROJECT_CONTENTWS_SERVICE_DICTIONARY")
                    contentWsUrl = siteMap.get(siteId, None)
                    logger.debug("Alternate site to contact is %r", contentWsUrl)
                    if contentWsUrl:
                        pD["session_proxy_url"] = contentWsUrl
                    else:
                        ok = False
                else:
                    # Look up with siteId info for depositions
                    pdbxFilePath = self.__fetchModelFile(siteId, entryId)
                    if os.access(pdbxFilePath, os.R_OK):
                        pD["session_pdbx_file_path"] = str(pdbxFilePath)
                    else:
                        ok = False

            elif contentType.startswith("report-summary-"):
                ok = True
                fName = contentType + "." + formatType

                # Check if our site can handle...
                qs = self._reqObj.getValueOrDefault("query_site", default=None)

                # Check if request is for a separate site and if can access
                if qs and qs != self._siteId and qs not in siteCoverage:
                    logger.error("Request for %s but not in %s or %s", qs, self._siteId, siteCoverage)
                    ok = False

                pD["query_site"] = qs

                logger.debug("Summary content type %r format type %r site %s status %r", contentType, formatType, qs, ok)

            if ok:
                resultPath = os.path.join(self._sessionPath, fName)
                pD["report_file"] = fName
                pD["report_path"] = resultPath

                # Not really used in this example -
                dstPathList = [resultPath]
                pD["dst_file_path_list"] = dstPathList
                #
                pD["session_path"] = self._sessionPath
                pD["session_store_prefix"] = self._sdsPrefix
                pD["session_history_path"] = self._reqObj.getSessionUserPath()
                pD["session_id"] = self._sessionId
                #
                logger.debug("Request payload %r", pD)

                pStatus = sD["status"]
                try:
                    # preset the status as the request may be serviced immediately
                    self._setSessionStoreValue("status", "submitted")
                    self._trackServiceStatus("submitted")
                    ok = self.__publishRequest(pD)
                    logger.debug("Publish method return status %r", ok)
                except Exception as e:
                    logger.exception("Failed publish method %s", str(e))
                    ok = False

                if not ok:
                    # restore the prior status
                    self._setSessionStoreValue("status", pStatus)

            if "session_history" in sD:
                shL = sD["session_history"]
            else:
                shL = []
            #
            if ok:
                sst.setAppDataDict(
                    {"session_id": self._sessionId, "session_history": shL},
                    format="json",
                )
                sst.setServiceCompletionFlag(ok)
                sst.setServiceStatusText("Submit successful")

            else:
                sst.setServiceCompletionFlag(False)
                sst.setServiceError(msg="Submit operation failed")
        #
        logger.debug("Completed request method")
        return sst

    def __publishRequest(self, pD):
        ok = False
        logger.debug("Publishing request with payload %r", pD)
        try:
            siteID = getSiteId()
            msg = json.dumps(pD)
            mp = MessagePublisher()
            ok = mp.publish(
                msg,
                exchangeName=get_exchange_name(),
                queueName=get_queue_name(site_id=siteID),
                routingKey=get_routing_key(),
            )
        except Exception as e:
            logger.exception("Failing publish request %s", str(e))
        return ok

    ##
    def _indexOp(self):
        """Returns an index of service output files -"""
        sst = ServiceSessionState()
        #            join an existing session -
        ok = self._getSession(new=False, useContext=True, contextOverWrite=True)
        if not ok:
            sst.setServiceCompletionFlag(ok)
            sst.setServiceError(msg="Session acquire failed")
        else:
            try:
                sD = self._getSessionStoreDict()
                if "session_history" in sD:
                    shL = sD["session_history"]
                else:
                    shL = []
                dD = {"session_id": self._sessionId, "session_history": shL}
                #
                rD = {}
                fpList = glob.glob(self._sessionPath + "/*")
                for fp in fpList:
                    if os.access(fp, os.R_OK):
                        _pth, fName = os.path.split(fp)
                        (_fn, ft) = os.path.splitext(fName)
                        rD[fName] = (fName, ft[1:])
                dD["index"] = rD
                sst.setAppDataDict(dD, format="json")
                sst.setServiceCompletionFlag(True)
                sst.setServiceStatusText("Index length %d" % len(rD))
            except Exception as e:
                logger.exception("I updating session history %s", str(e))
                dD["index"] = rD
                sst.setAppDataDict(dD, format="json")
                sst.setServiceCompletionFlag(True)
                sst.setServiceError(msg="Index is empty")

        logger.debug("Completed index method")
        return sst

    ##
    def _downloadOp(self):
        """Download a file in the current session by type or by name -"""
        logger.debug("Starting now")
        sst = ServiceSessionState()
        try:
            #     - join an existing session -
            ok = self._getSession(new=False, useContext=True, contextOverWrite=True)
            if not ok:
                sst.setServiceCompletionFlag(ok)
                sst.setServiceError(msg="Session acquire failed")
            else:
                ct = self._reqObj.getValue("contenttype")
                sD = self._getSessionStoreDict()
                logger.debug("Session store keys %r ", sD.keys())
                logger.debug("Content type %r ", ct)
                #
                if ct in sD:
                    fn, _fmt = sD[ct]
                else:
                    fn = self._reqObj.getValue("filename")

                fp = os.path.join(self._sessionPath, fn)
                logger.debug("download target path %r", fp)
                md5Digest = getMD5(fp, block_size=4096, hr=True)
                sst.setDownload(fn, fp, contentType=None, md5Digest=md5Digest)
                # self._trackServiceStatus('downloading', {'file': fn})
                self._trackServiceStatus("downloading", file=fn)
        except Exception as e:
            logger.exception("FAILING %s", str(e))
            sst.setServiceCompletionFlag(False)
            sst.setServiceError(msg="File download failed")

        return sst

    def _activityOp(self):
        """Returns a summary of service activity  - for authenticated sessions -"""
        sst = ServiceSessionState()
        #            join an existing session -
        ok = self._getSession(new=False, useContext=True, contextOverWrite=True)
        if not ok:
            sst.setServiceCompletionFlag(ok)
            sst.setServiceError(msg="Session acquire failed")
        else:
            if self._reqObj.getValue("service_user_id") == "CONTENTWS_ANONYMOUS":
                dD = {
                    "session_id": self._sessionId,
                    "session_history": [],
                    "activity_summary": {},
                }
                sst.setAppDataDict(dD, format="json")
                sst.setServiceCompletionFlag(True)
                sst.setServiceError(msg="Activity history is not available")
            else:
                try:
                    sD = self._getSessionStoreDict()
                    if "session_history" in sD:
                        shL = sD["session_history"]
                    else:
                        shL = []
                    dD = {"session_id": self._sessionId, "session_history": shL}
                    #
                    dD["activity_summary"] = self._getServiceActivitySummary()
                    sst.setAppDataDict(dD, format="json")
                    sst.setServiceCompletionFlag(True)
                    sst.setServiceStatusText("Activity history length %d" % len(dD["activity_summary"]))
                except Exception as e:
                    logger.exception("Updating session store dict %s", str(e))
                    dD["activity_summary"] = {}
                    sst.setAppDataDict(dD, format="json")
                    sst.setServiceCompletionFlag(True)
                    sst.setServiceError(msg="Activity history is empty")

        logger.debug("Completed history summary method")
        return sst

    def __runTimeout(self, cmdFile=None, logFile=None, timeout=10):
        """Execute the command as a subprocess with a timeout."""
        logger.debug("STARTING with time out set at %d (seconds)", timeout)
        #
        start = datetime.datetime.now()
        try:
            process = Popen(  # pylint: disable=subprocess-popen-preexec-fn
                cmdFile,
                stdout=PIPE,
                stderr=PIPE,
                shell=False,
                close_fds=True,
                preexec_fn=os.setsid,
            )
            while process.poll() is None:
                time.sleep(0.1)
                now = datetime.datetime.now()
                if (now - start).seconds > timeout:
                    os.killpg(process.pid, signal.SIGKILL)
                    os.waitpid(-1, os.WNOHANG)
                    logger.debug("Execution terminated by timeout %d (seconds)", timeout)
                    if logFile is not None:
                        ofh = open(logFile, "a")
                        ofh.write("Execution terminated by timeout %d (seconds)\n" % timeout)
                        ofh.close()
                    #
                    return None
                #
                #
        except Exception as e:
            logger.error("Exception", exc_info=True)
            logger.error(e)
        #
        output = process.communicate()
        logger.debug("completed with stdout data %r", output[0])
        logger.debug("completed with stderr data %r", output[1])
        logger.debug("completed with return code %r", process.returncode)
        return process.returncode

    #



##
# File:  ContentRequestPolicyFilter.py
# Date:  05-Jul-2017  E. Peisach
#
# Update:
##
"""
Policy filtering of content request for web service -

"""
__docformat__ = "restructuredtext en"
__author__ = "Ezra Peisach"
__email__ = "peisach@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import logging

#
logger = logging.getLogger()


class ContentRequestPolicyFilter(object):
    """
    Manage various policy filters for entry data. A post request filter.

    """

    def __init__(self):
        pass

    def filterContent(self, contentType, rD):
        """Filters rD based on contentType.  Returns new content or {} if all removed"""

        if not rD:
            return rD

        logger.debug("Content in %r", rD)
        if "sasbdb" in contentType:
            # Policy 1: suppress title and author
            pDS = rD.get("pdbx_database_status", None)
            if pDS:
                logger.debug("Filtering %r", pDS)
                tS = pDS[0].get("title_suppression", None)
                if tS and tS in ["Y"]:
                    logger.debug("Removing author and title and contact")
                    rD["audit_author"] = [{}]
                    rD["pdbx_contact_author"] = [{}]
                    rD["struct"][0]["title"] = ""
            # Policy 2: If not sas - you get nothing
            removeAll = True
            pDSF = rD.get("pdbx_depui_status_flags", None)
            if pDSF:
                hSD = pDSF[0].get("has_sas_data", None)
                if hSD and hSD in ["Y"]:
                    removeAll = False
            if removeAll:
                rD = {}

        logger.debug("Content out %r", rD)

        return rD



##
# File:  ContentRequestProxyReportPdbx.py
# Date:  02-Jun-2017  E. Peisach. Westbrook
#
# Update:
##
"""
Fetch content and prepare report from PDBx content -

"""
__docformat__ = "restructuredtext en"
__author__ = "Ezra Peisach"
__email__ = "peisach@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import logging
import time

from onedep_biocuration.api.ContentRequest import ContentRequest
from wwpdb.utils.config.ConfigInfo import ConfigInfo, getSiteId

logger = logging.getLogger()


class ContentRequestProxyReportPdbx(object):
    """
    Fetch content and prepare report from PDBx content from a remote service using
    biocuration api.

    """

    def __init__(self, verbose=True):  # pylint: disable=unused-argument
        # self.__verbose = verbose
        self.__siteId = getSiteId(defaultSiteId=None)
        self.__cI = ConfigInfo(self.__siteId)
        # Max time to get a response
        self.__maxWait = 60
        logger.info("Starting with siteId %r", self.__siteId)
        #

    def __readApiKey(self, filePath):
        """Reads API key from file"""
        api_key = None
        try:
            with open(filePath, "r") as fp:
                api_key = fp.read()
        except Exception as e:
            logger.exception(e)

        return api_key

    def retrieveProxyReport(self, dataSetId, apiUrl, contentType, formatType, reportPath):
        """Retrieve a report from a remote server"""
        logger.debug("dataSetId %r apiUrl %r contentType %r reportPath %r", dataSetId, apiUrl, contentType, reportPath)

        apiKeyFileName = self.__cI.get("SITE_WS_CONTENT_WWPDB_KEY")
        apiKey = self.__readApiKey(apiKeyFileName)
        if not apiKey:
            logger.error("Could not read apiKeyFile %r", apiKeyFileName)
            return False

        cr = ContentRequest(apiKey=apiKey, apiUrl=apiUrl)

        # Need a session....
        rD = cr.createSession()
        if not rD or rD["onedep_error_flag"]:
            logger.error("Response from create session %r", rD)
            return False

        sessionId = rD["session_id"]
        logger.debug("sessionId %r", sessionId)

        rD = cr.requestEntryContent(dataSetId, contentType, formatType)
        if rD["onedep_error_flag"]:
            logger.error("Submitted content service failed request %r", rD)
            return False

        logger.debug("Submitted remote content reuqest")

        #
        #   Poll for service completion -
        #
        total = 0
        it = 0
        sl = 2
        while True:
            #    Pause -
            it += 1
            pause = it * it * sl
            time.sleep(pause)
            rD = cr.getStatus()
            if rD["status"] in ["completed", "failed"]:
                break
            logger.debug("[%4d] Pausing for %4d (seconds)", it, pause)
            total += pause
            # Timeout?
            if pause > self.__maxWait:
                logger.error("No response from remote service in %r", total)
                return False
        #
        logger.debug("Received response from remote %r", rD)

        if rD["status"] == "failed" or rD["onedep_error_flag"]:
            logger.error("Remote service request failed %r", rD)
            return False

        rD = cr.getOutputByType(reportPath, contentType, formatType=formatType)
        if rD["onedep_error_flag"]:
            logger.debug("getOutputByType failed %r", rD)
            return False

        # We have succeeded!!!!
        return True



##
# File:  WebServiceApp.py
# Date:  3-July-2016 Jdw
#
# Updates:
#    7-Feb-2017  jdw adapt for content provider service
#
##
"""
Manage web request and response processing for various web services.

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import logging

#
from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.utils.ws_utils.ServiceRequest import ServiceRequest
from wwpdb.utils.ws_utils.ServiceResponse import ServiceResponse

#
from wwpdb.apps.content_ws_server.webapp.ContentServiceAppWorker import (
    ContentServiceAppWorker,
)

logger = logging.getLogger()


class WebServiceApp(object):
    """Handle request and response object processing for various web services."""

    def __init__(self, parameterDict=None):
        """
        Create an instance of the appropriate worker class to manage input service request.

         :param `parameterDict`: dictionary storing parameter information from the input request.

        """
        if parameterDict is None:
            parameterDict = {}
        self.__reqObj = ServiceRequest(parameterDict)
        siteId = self.__reqObj.getSiteId()
        #
        cI = ConfigInfo(siteId)
        self.__reqObj.setTopSessionPath(cI.get("SITE_WEB_APPS_TOP_SESSIONS_PATH"))
        self.__reqObj.setRequestPathPrefix(cI.get("SITE_WEB_SERVICE_PATH_PREFIX", default="/service"))
        self.__reqObj.setDefaultReturnFormat(return_format="json")
        #

    def run(self):
        """Execute request and package results in response dictionary.

        :Returns:
             A dictionary containing response data for the input request.
             Minimally, the content of this dictionary will include the
             keys: CONTENT_TYPE and REQUEST_STRING.
        """

        requestPath = self.__reqObj.getRequestPath()
        logger.debug("Processing requiest path : %s", requestPath)
        #
        if requestPath.startswith("/contentws"):
            swrk = ContentServiceAppWorker(reqObj=self.__reqObj)
        else:
            swrk = ContentServiceAppWorker(reqObj=self.__reqObj)
        #
        #  Each class implements a run() method that returns a ServiceSessionState object -
        #
        sst = swrk.run()
        logger.debug("Service response object for request path: %s\n   %r", requestPath, sst.getAppDataDict())
        sr = self.__buildResponse(sst)
        #
        # Return only the dictionary from the response object -
        #
        return sr

    def __buildResponse(self, sst):
        """Using the content from the input service state object, build a service response object."""
        rD = {}
        sr = ServiceResponse(returnFormat="json", injectStatus=False)
        #
        if sst.getServiceErrorFlag():
            rD["errorflag"] = True
            rD["statusmessage"] = sst.getServiceErrorMessage()
            sr.setData(rD)
        elif sst.getResponseFormat() in ["json"]:
            rD["errorflag"] = False
            sm = sst.getServiceStatusText()
            if sm and len(sm) > 0:
                rD["statusmessage"] = sm
            else:
                rD["statusmessage"] = "ok"
            #
            rD.update(sst.getAppDataDict())
            sr.setData(rD)
        elif sst.getResponseFormat() in ["files"]:
            sr.setReturnFormat("binary")
            fL = sst.getDownloadList()
            ok = False
            for f in fL[0:1]:
                # f = (fileName, filePath, contentType, md5Digest)
                ok = sr.setBinaryFile(f[1], attachmentFlag=False, serveCompressed=True, md5Digest=f[3])
            if not ok:
                sr.setReturnFormat("json")
                rD["errorflag"] = True
                rD["statusmessage"] = "Download file not found"
                sr.setData(rD)
        else:
            rD["errorflag"] = True
            rD["statusmessage"] = "Miscellaneous error"
            sr.setData(rD)
        #
        #
        return sr
